package com.icontrols.kakao.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CompletableFuture;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.icontrols.kakao.common.KAKAO_RESULT;
import com.icontrols.kakao.common.KEY_INFO;
import com.icontrols.kakao.common.utils.CommonUtil;
import com.icontrols.kakao.common.utils.JWTUtils2;
import com.icontrols.kakao.config.AsyncConfig;
import com.icontrols.kakao.entity.DonghoInfo;
import com.icontrols.kakao.repo.DonghoInfoRepository;
import com.icontrols.kakao.service.KakaoExtensionService;
import com.icontrols.kakao.socket.wallpad.KakaoMsgHandler;
import com.icontrols.kakao.socket.wallpad.WallpadMsgSender;

@Service
public class KakaoExtensionServiceImpl implements KakaoExtensionService {
	private final Logger logger = LoggerFactory.getLogger(KakaoExtensionService.class);
	
	@Resource(name="asyncConfig")
	private AsyncConfig asyncConfig;
	
	@Resource(name="kakaoMsgHandler")
	KakaoMsgHandler kMsgHandler;
	
	@Resource(name="wallpadMsgSender")
	WallpadMsgSender wallpadMsgSender;
	
	@Autowired
	DonghoInfoRepository donghoRepository;
	
	@Override
	public Map<String, Object> request_grantAuthorization(Map<String, Object> param) throws Exception {
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
//		int dong = Integer.parseInt(dong_ho.split("/")[0]);
//		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		int dong = 802;
		int ho = 1103;
		
		Map<String, Object> body = (HashMap)param.get(KEY_INFO.HTTP_KEY_BODY);
		
		try {
			DonghoInfo donghoInfo = new DonghoInfo(
					Integer.toString(dong), 
					Integer.toString(ho), 
					(String) body.get(KEY_INFO.BODY_KEY_ACCESS_TOKEN),
					(String) body.get(KEY_INFO.BODY_KEY_REFRESH_TOKEN),
					Integer.toString((int)body.get(KEY_INFO.BODY_KEY_EXPIRES_IN)),
					CommonUtil.getStrTypeCurTime(),
					CommonUtil.getStrTypeCurTimeAfterSec((int)body.get(KEY_INFO.BODY_KEY_EXPIRES_IN)));
			
//			List<DonghoInfo> dList = donghoRepository.findByDongAndHo(donghoInfo.getDong(), donghoInfo.getHo());
			List<DonghoInfo> dList = donghoRepository.findByKakaoAccessTokenAndKakaoRefreshToken(
																									(String) body.get(KEY_INFO.BODY_KEY_ACCESS_TOKEN), 
																									(String) body.get(KEY_INFO.BODY_KEY_REFRESH_TOKEN)
																								);
			
			if(dList.size() > 0)
				donghoInfo.setId(dList.get(0).getId());

			donghoRepository.save(donghoInfo);
			
			logger.debug("dongho info saved complete " );
			logger.debug(donghoInfo.toString());
			
		}catch(Exception e) {
			logger.error("[Auth Database Insert Error]");
			e.printStackTrace();
			
			param.put(KEY_INFO.HTTP_KEY_ERROR, KAKAO_RESULT.getErrorInfo(KAKAO_RESULT.FAIL_INTERNAL));
		}finally {
			param.remove(KEY_INFO.HTTP_KEY_BODY);
		}
		return param;
	}
	
	@Override
	public Map<String, Object> request_discover(Map<String, Object> param) throws Exception{
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
//		int dong = Integer.parseInt(dong_ho.split("/")[0]);
//		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		int dong = 802;
		int ho = 1103;
		
		Map<String, Object> listMap = kMsgHandler.getDevicesStatusList(dong, ho);
		param.putAll(listMap);
		return param;
	}
	
	@Override
	public Map<String, Object> request_queryState(Map<String, Object> param) throws Exception {
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
//		int dong = Integer.parseInt(dong_ho.split("/")[0]);
//		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		int dong = 802;
		int ho = 1103;
		Map <String, Object> body = (HashMap)param.get(KEY_INFO.HTTP_KEY_BODY);
		
		List<String> endpointIds = (ArrayList)body.get(KEY_INFO.BODY_KEY_ENDPOINT_IDS);
		List<String> unreachableIds = new ArrayList<String>();
		
		List<CompletableFuture<Map<String, Object>>> fList = new ArrayList<>();
		
		body.clear();
		for(String endpointId:endpointIds) {
			while(true) {
    			if(asyncConfig.isTaskExecute()) {
    				fList.add(wallpadMsgSender.getDeviceStatus(dong, ho, endpointId));
    				break;
    			}
    			Thread.sleep(1000);
    		}
		}
		
		
		CompletableFuture.allOf(fList.toArray(new CompletableFuture[fList.size()])).join();
		
		logger.debug("******************************************************************************");
        logger.debug("								ASYNC D END");
        logger.debug("******************************************************************************");
		
		for(CompletableFuture<Map<String, Object>> future:fList) {
        	Map<String, Object> retMap = future.get();
        	
        	String endpointId = (String)retMap.get(KEY_INFO.BODY_KEY_ENDPOINT_ID);
        	logger.debug(endpointId + "-------->" + retMap.toString());
        	
        	if(retMap.containsKey(KEY_INFO.HTTP_KEY_ERROR)) {
				unreachableIds.add(endpointId);
				
				HashMap<String, Object> tmpMap = new HashMap<String, Object>();
				tmpMap.put("power", "off");
				
				retMap.clear();
				retMap.put(endpointId, tmpMap);
			}
        	
        	retMap.remove(KEY_INFO.BODY_KEY_ENDPOINT_ID);
        	
        	body.putAll(retMap);
        }
		
		param.put(KEY_INFO.HTTP_KEY_UNREACHABLEIDS, unreachableIds);
		param.put(KEY_INFO.HTTP_KEY_BODY, body);
		
		return param;
	}
	
	
	@Override
	public Map<String, Object> request_updateState(Map<String, Object> param) throws Exception {
		String dong_ho = JWTUtils2.getHomeIdFromToken((String)param.get(KEY_INFO.HTTP_KEY_AUTHORIZATION));
//		int dong = Integer.parseInt(dong_ho.split("/")[0]);
//		int ho = Integer.parseInt(dong_ho.split("/")[1]);
		
		int dong = 802;
		int ho = 1103;
		
		@SuppressWarnings("unchecked")
		Map <String, Object> body = (HashMap<String, Object>)param.get(KEY_INFO.HTTP_KEY_BODY);
		
		@SuppressWarnings("unchecked")
		Map<String, Object> desired = (HashMap<String, Object>)body.get(KEY_INFO.BODY_KEY_DESIRED);
		
		String endpointId = (String)body.get(KEY_INFO.BODY_KEY_ENDPOINT_ID);
		String capabilityName = (String)body.get(KEY_INFO.BODY_KEY_CAPABILITY_NAME);
		
		param.put(KEY_INFO.HTTP_KEY_BODY, kMsgHandler.sendDeviceAction(dong, ho, endpointId, capabilityName, desired));
		return param;
	}
}
